-------------------------------------------------------------------------------
--
-- Title       : FIFO_Buffer
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description :  FIFO made of Dual-Port RAM with One Enable Controlling Both Ports
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

-------------------------------------------------------------------------------

entity FIFO_Buffer is
	generic (
		constant DEPTH : integer := 8;				-- size of the BRAM
		constant FIFO_LENGTH : integer := 256		-- length of the FIFO
		);
	port(
		CLK  : in  std_logic;
		CLR  : in  std_logic;
		EN   : in  std_logic;
		DIN  : in  std_logic_vector(7 downto 0);
		DOUT : out std_logic_vector(7 downto 0)
		);
end FIFO_Buffer;

-------------------------------------------------------------------------------

architecture FIFO_Buffer_Arch of FIFO_Buffer is
	
	-- signals for address counter
	signal addrA : std_logic_vector(DEPTH-1 downto 0);
	signal addrB : std_logic_vector(DEPTH-1 downto 0);
	
	-- auxiliary signal for output A
	signal doA : std_logic_vector(7 downto 0);
	
	---------------------------------------------------------------------------
	
	component DP_Ram_1xE is
		generic (
			constant DEPTH : integer
			);
		port (
			clk : in std_logic;
			en : in std_logic;
			we : in std_logic;
			addra : in std_logic_vector(DEPTH-1 downto 0);
			addrb : in std_logic_vector(DEPTH-1 downto 0);
			di : in std_logic_vector(7 downto 0);
			doa : out std_logic_vector(7 downto 0);
			dob : out std_logic_vector(7 downto 0)
			);
	end component DP_Ram_1xE;
	
	---------------------------------------------------------------------------
begin
	---------------------------------------------------------------------------
	
	U1: DP_Ram_1xE generic map (DEPTH => DEPTH)
	port map (
		clk => CLK,
		en => EN,
		we => EN,
		addra => addrA,
		addrb => addrB,
		di => DIN,
		doa => doA,
		dob => DOUT
		);	
	
	---------------------------------------------------------------------------
	
	AddrA_Gen_Proc: process (CLK,CLR) is
	begin
		if CLR = '1' then
			addrA <= (others => '0');
		elsif CLK'event and CLK = '1' then
			if EN = '1' then
--	  naley uzupeni sposb adresacji 
--
--

			end if;
		end if;
	end process AddrA_Gen_Proc;
	
	---------------------------------------------------------------------------
	
	AddrB_Gen_Proc: process (CLK,CLR) is
	begin
		if CLR = '1' then
			addrB <= (others => '0');
		elsif CLK'event and CLK = '1' then
			if EN = '1' then
--	  naley uzupeni sposb adresacji 
--
--
			end if;
		end if;
	end process AddrB_Gen_Proc;
	
	---------------------------------------------------------------------------
	
end FIFO_Buffer_Arch;
