-------------------------------------------------------------------------------
--
-- Title       : Send_Image
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description : This module puts samples into Matlab's array
--
-------------------------------------------------------------------------------

library IEEE; 	  
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

library aldec;
use aldec.matlab.all;

-------------------------------------------------------------------------------

entity Send_Image is
	generic (
		 V_IM_DIM  : integer;			-- size of vertical dimension
		 H_IM_DIM  : integer;			-- size of horizontal dimension
		 D_IM_DIM  : integer;			-- depth of image
		 NO_FRAMES : integer;			-- number of frames
		 PIPELINE_LENGTH : integer	-- length of pipeline in filter procedure
		);
	port(
		CLK : in STD_LOGIC;
		CLR : in STD_LOGIC;
		EN : in STD_LOGIC;
		DATA_IN : in STD_LOGIC_VECTOR(7 downto 0)
		);
end Send_Image;

-------------------------------------------------------------------------------

architecture Send_Image of Send_Image is 
	
begin
	-------------------------------------------------------------------------------
	-- Main_Proc process
	-- It reads  and sends the samples
	-----------------------------------------------------------------------------
	Main_Proc: process (CLK,CLR,EN)
		
		------------------------------------------------------------------------
		-- auxiliary variables
		variable run_once	  : boolean := false;	  			-- auxiliary flag to init data reading
		variable vcounter 	  : integer range 1 to V_IM_DIM;  	-- vertical counter
		variable hcounter 	  : integer range 1 to H_IM_DIM;	-- horizontal counter
		variable dcounter 	  : integer range 1 to D_IM_DIM;	-- depth counter
		variable fcounter	  : integer range 1 to NO_FRAMES;	-- frame counter
		variable out_array_id : integer range 0 to 2;  			-- data sample array ID
		
		variable m_valid	  : boolean := false;				-- flag of valid sample on the input
		-- auxiliary counter for valid sample flag
		variable pixel_count  : integer range 1 to 2*H_IM_DIM+3+PIPELINE_LENGTH := 1;
		
		------------------------------------------------------------------------
	begin
		------------------------------------------------------------------------
		-- this part of the process should runs just once
		if(run_once = false) then
			run_once := true;
			ml_setup(false, 0, mxUINT8, mxUNSIGNED);
			out_array_id := create_array("img_out", 4, (V_IM_DIM-2, H_IM_DIM-2, D_IM_DIM, NO_FRAMES) );
		end if;
		
		if CLR = '1' then
			vcounter := 1;
			hcounter := 1;
			dcounter := 1;
			fcounter := 1;
			m_valid := false;
			pixel_count := 1;
		elsif rising_edge(CLK) then
			if EN = '1' then	
				-- naley dopisa odpowiednie warunki dla hcounter i vcounter
				-- tak aby zapisywane byy tylko poprawne piksele
				if ((hcounter < H_IM_DIM-1) and (vcounter < V_IM_DIM-1) and (m_valid = true))then
					-- put data into MATLAB variable
					put_item(DATA_IN, mxUNSIGNED, out_array_id, (vcounter, hcounter, dcounter, fcounter));
				end if;	
				
				-- set dimensions
				if m_valid = true then
					if hcounter = H_IM_DIM then
						hcounter := 1;
						if vcounter = V_IM_DIM then
							vcounter := 1;
							if dcounter = D_IM_DIM then
								dcounter := 1;
								if fcounter = NO_FRAMES then
									-- send MATLAB object array
									hdl2ml( out_array_id );
									-- execute function toc
									eval_string("toc");
									-- run MATLAB M-file
									eval_string( "write_output_image" );
									-- release memory 
									destroy_array( out_array_id );
								else
									fcounter := fcounter + 1;
								end if;
							else
								dcounter := dcounter + 1;
							end if;
						else
							vcounter := vcounter+1;
						end if;
					else
						hcounter := hcounter+1;
					end if;
				end if;
				
				-- wait for valid pixel		

				if pixel_count < 2*H_IM_DIM+3+PIPELINE_LENGTH then
					pixel_count := pixel_count + 1;
				else
					m_valid := true;					
				end if;
				
			end if;
		end if;	
	end process Main_Proc;
	
end Send_Image;
